# -*- coding: utf-8 -*-
"""
@author: Sam Schott  (ss2151@cam.ac.uk)

(c) Sam Schott; This work is licensed under a Creative Commons
Attribution-NonCommercial-NoDerivs 2.0 UK: England & Wales License.

"""
import os
from enum import Enum


class SupportedImplementation(Enum):
    notify_send = 'notify-send'
    osascript = 'osascript'


class Notipy(object):
    """Send native OS notifications to user.

    Relies on AppleScript on macOS and notify-send on linux, otherwise
    falls back to stdout."""

    enabled = True

    def __init__(self):
        self.implementation = self.__get_available_implementation()

    def send(self, message, title="CustomXepr"):
        if self.enabled:
            self.__send_message(message, title)
        else:
            pass

    def __send_message(self, message, title=""):
        if self.implementation == SupportedImplementation.osascript:
            os.system("osascript -e 'display notification \"{}\" with title \"{}\"'".format(
                message, title))
        elif self.implementation == SupportedImplementation.notify_send:
            os.system('notify-send "{}" "{}"'.format(title, message))
        else:
            print('{}: {}'.format(title, message))

    @staticmethod
    def __command_exists(command):
        return any(
            os.access(os.path.join(path, command), os.X_OK)
            for path in os.environ["PATH"].split(os.pathsep)
        )

    def __get_available_implementation(self):
        if self.__command_exists('osascript'):
            return SupportedImplementation.osascript
        elif self.__command_exists('notify-send'):
            return SupportedImplementation.notify_send
        return None



# -*- coding: utf-8 -*-
"""
@author: Sam Schott  (ss2151@cam.ac.uk)

(c) Sam Schott; This work is licensed under a Creative Commons
Attribution-NonCommercial-NoDerivs 2.0 UK: England & Wales License.

"""

from PyQt5 import QtGui, QtWidgets


class ListValidator(QtGui.QValidator):
    """
    This is a validator for a list of float values.
    """

    accepted_strings = []

    def validate(self, string, position):
        """
        This is the actual validator. It checks whether the current user input is a valid string
        every time the user types a character. There are 3 states that are possible.
        1) Invalid: The current input string is invalid. The user input will not accept the last
                    typed character.
        2) Acceptable: The user input in conform with the regular expression and will be accepted.
        3) Intermediate: The user input is not a valid string yet but on the right track. Use this
                         return value to allow the user to type fill-characters needed in order to
                         complete an expression (i.e. the decimal point of a float value).
        :param string: The current input string (from a QLineEdit for example)
        :param position: The current position of the text cursor
        :return: enum QValidator::State: the returned validator state,
                 str: the input string, int: the cursor position
        """

        string_list = string.split(',')
        validated_list = [self.validate_string(x) for x in string_list]

        if self.Invalid in validated_list:
            return self.Invalid, string, position
        elif self.Intermediate in validated_list:
            return self.Intermediate, string, position
        else:
            return self.Acceptable, string, position

    def validate_string(self, text):

        text = text.strip()
        text = text.rstrip()

        try:
            float(text)
            return self.Acceptable
        except ValueError:

            if text in ['', '-', '+']:
                return self.Intermediate

            for string in self.accepted_strings:
                if string == text:
                    return self.Acceptable
                elif string.startswith(text):
                    return self.Intermediate

            return self.Invalid


class FloatListWidget(QtWidgets.QLineEdit):

    _accepted_strings = []

    def __init__(self, *args, **kwargs):
        """"
        A QLineEdit which only allows comma separated lists to be entered. By default, the
        values must be numeric but :func:`setAcceptedStrings` allows the user to set a
        list of string values which will also be accepted.
        """
        super(FloatListWidget, self).__init__(*args, **kwargs)

        self.validator_ = ListValidator()
        self.validator_.accepted_strings = self._accepted_strings
        self.setValidator(self.validator_)

    def value(self):
        """
        Return the current list of values.

        :return: List of values.
        :rtype: list
        """
        text = self.text()
        string_list = text.split(',')

        return [self._string_to_value(x) for x in string_list]

    def setValue(self, value_list):
        """
        Set the current value.

        :param list value_list: List of values.
        """

        string_list = []

        for value in value_list:
            if value in self._accepted_strings:
                string_list.append(value)
            elif isinstance(value, float) and value.is_integer():
                string_list.append(str(int(value)))
            else:
                string_list.append(str(value))

        string = ', '.join(string_list)

        string = string.replace('  ', ' ')
        string = string.strip()

        self.setText(string)

    def acceptedStrings(self):
        """
        Returns a list of accepted strings.

        :return: List of accepted strings.
        :rtype: list
        """
        return self._accepted_strings

    def setAcceptedStrings(self, string_list):
        """
        Accepts the string given in :param:`string_list` as input values.

        :param list string_list: list of strings.
        """

        if not all([isinstance(x, str) for x in string_list]):
            raise ValueError('Input must be a list of strings.')

        self._accepted_strings = string_list
        self.validator_.accepted_strings = string_list

    def _string_to_value(self, string):
        try:
            return float(string)
        except ValueError:
            if string in self._accepted_strings:
                return string
            else:
                raise ValueError('Invalid drain voltage.')



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: Sam Schott  (ss2151@cam.ac.uk)

(c) Sam Schott; This work is licensed under a Creative Commons
Attribution-NonCommercial-NoDerivs 2.0 UK: England & Wales License.

"""
# external packages
from PyQt5 import QtCore, QtWidgets
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPainter, QPixmap, QWindow


class FaderWidget(QtWidgets.QWidget):

    pixmap_opacity = 1.0

    def __init__(self, old_widget, new_widget, duration=300):
        QtWidgets.QWidget.__init__(self, new_widget)

        pr = QWindow().devicePixelRatio()
        self.old_pixmap = QPixmap(new_widget.size()*pr)
        self.old_pixmap.setDevicePixelRatio(pr)
        old_widget.render(self.old_pixmap)

        self.timeline = QtCore.QTimeLine()
        self.timeline.valueChanged.connect(self.animate)
        self.timeline.finished.connect(self.close)
        self.timeline.setDuration(duration)
        self.timeline.start()

        self.resize(new_widget.size())
        self.show()

    def paintEvent(self, event):
        painter = QPainter()
        painter.begin(self)
        painter.setOpacity(self.pixmap_opacity)
        painter.drawPixmap(0, 0, self.old_pixmap)
        painter.end()

    def animate(self, value):
        self.pixmap_opacity = 1.0 - value
        self.repaint()


class AnimatedStackedWidget(QtWidgets.QStackedWidget):
    """
    A subclass of ``QStackedWidget`` with sliding or fading animations between stacks.
    """

    def __init__(self, parent=None):
        super(AnimatedStackedWidget, self).__init__(parent)

        self.m_direction = Qt.Horizontal
        self.m_speed = 300
        self.m_animationtype = QtCore.QEasingCurve.OutCubic
        self.m_now = 0
        self.m_next = 0
        self.m_wrap = False
        self.m_pnow = QtCore.QPoint(0, 0)
        self.m_active = False

    def setDirection(self, direction):
        self.m_direction = direction

    def setSpeed(self, speed):
        self.m_speed = speed

    def setAnimation(self, animationtype):
        self.m_animationtype = animationtype

    def setWrap(self, wrap):
        self.m_wrap = wrap

    @QtCore.pyqtSlot()
    def slideInPrev(self):
        now = self.currentIndex()
        if self.m_wrap or now > 0:
            self.slideInIdx(now - 1)

    @QtCore.pyqtSlot()
    def slideInNext(self):
        now = self.currentIndex()
        if self.m_wrap or now < (self.count() - 1):
            self.slideInIdx(now + 1)

    def slideInIdx(self, idx):
        if idx > (self.count() - 1):
            idx = idx % self.count()
        elif idx < 0:
            idx = (idx + self.count()) % self.count()
        self.slideInWgt(self.widget(idx))

    def slideInWgt(self, newwidget):
        if self.m_active:
            return

        self.m_active = True

        _now = self.currentIndex()
        _next = self.indexOf(newwidget)

        if _now == _next:
            self.m_active = False
            return

        offsetx, offsety = self.frameRect().width(), self.frameRect().height()
        self.widget(_next).setGeometry(self.frameRect())

        if not self.m_direction == Qt.Horizontal:
            if _now < _next:
                offsetx, offsety = 0, -offsety
            else:
                offsetx = 0
        else:
            if _now < _next:
                offsetx, offsety = -offsetx, 0
            else:
                offsety = 0

        pnext = self.widget(_next).pos()
        pnow = self.widget(_now).pos()
        self.m_pnow = pnow

        offset = QtCore.QPoint(offsetx, offsety)
        self.widget(_next).move(pnext - offset)
        self.widget(_next).show()
        self.widget(_next).raise_()

        anim_group = QtCore.QParallelAnimationGroup(
            self, finished=self.animationDoneSlot
        )

        for index, start, end in zip((_now, _next), (pnow, pnext - offset), (pnow + offset, pnext)):
            animation = QtCore.QPropertyAnimation(
                self.widget(index),
                b"pos",
                duration=self.m_speed,
                easingCurve=self.m_animationtype,
                startValue=start,
                endValue=end,
            )
            anim_group.addAnimation(animation)

        self.m_next = _next
        self.m_now = _now
        self.m_active = True
        anim_group.start(QtCore.QAbstractAnimation.DeleteWhenStopped)

    @QtCore.pyqtSlot()
    def animationDoneSlot(self):
        self.setCurrentIndex(self.m_next)
        self.widget(self.m_now).hide()
        self.widget(self.m_now).move(self.m_pnow)
        self.m_active = False

    def fadeInIdx(self, index):
        self.fader_widget = FaderWidget(self.currentWidget(), self.widget(index),
                                        self.m_speed)
        self.setCurrentIndex(index)


class AnimatedResizeWidget(QtWidgets.QWidget):

    def adjustSize(self):
        self.animatedResize()

    def animatedResize(self, newGeometry=None, ms_time=200):
        currentGeometry = self.geometry()
        if not newGeometry:
            newSize = self._adjustedSize()
            newGeometry = QtCore.QRect(currentGeometry.topLeft(), newSize)

        self.animation = QtCore.QPropertyAnimation(self, b"geometry")
        self.animation.setDuration(ms_time)
        self.animation.setStartValue(currentGeometry)
        self.animation.setEndValue(newGeometry)
        self.animation.start()

    def _adjustedSize(self):

        s = self.sizeHint()

        if self.isWindow():
            layout = self.layout()

            if layout:
                if layout.hasHeightForWidth():
                    s.setHeight(layout.totalHeightForWidth(s.width()))
                exp = layout.expandingDirections()
            else:
                if self.sizePolicy().hasHeightForWidth():
                    s.setHeight(self.heightForWidth(s.width()))
                exp = self.sizePolicy().expandingDirections()

            if int(exp) & QtCore.Qt.Horizontal:
                s.setWidth(max(s.width(), 200))
            if int(exp) & QtCore.Qt.Vertical:
                s.setHeight(max(s.height(), 100))

            screen = QtWidgets.QDesktopWidget().screenGeometry(self.pos())

            s.setWidth(min(s.width(), screen.width()*2/3))
            s.setHeight(min(s.height(), screen.height()*2/3))

        if not s.isValid():
            r = self.childrenRect()
            if r.isNull():
                return r
            s = r.size() + QtCore.QSize(2 * r.x(), 2 * r.y())

        return s



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Oct 31 16:23:13 2018

@author: samschott
"""

from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import Qt, QRect
from PyQt5.QtGui import QBrush, QImage, QPainter, QPixmap, QWindow


_USER_DIALOG_ICON_SIZE = 70


def elide_string(string, font=None, pixels=200, side="right"):
    """
    Elides a string to fit into the given width.

    :param str string: String to elide.
    :param font: Font to calculate size. If not given, the current style's default font
        for a QLabel is used.
    :param int pixels: Maximum width in pixels.
    :param str side: Side to truncate. Can be "right" or "left", defaults to "right".
    :return: Truncated string.
    :rtype: str
    """

    if not font:
        font = QtWidgets.QLabel().font()

    metrics = QtGui.QFontMetrics(font)
    mode = Qt.ElideRight if side is "right" else Qt.ElideLeft

    return metrics.elidedText(string, mode, pixels)


def get_scaled_font(scaling=1.0, bold=False, italic=False):
    """
    Returns the current style's default font for a QLabel but scaled by the given factor.

    :param float scaling: Scaling factor.
    :param bool bold: Sets the returned font to bold (defaults to ``False``)
    :param bool italic: Sets the returned font to italic (defaults to ``False``)
    :return: `QFont`` instance.
    """
    label = QtWidgets.QLabel()
    font = label.font()
    font.setBold(bold)
    font.setItalic(italic)
    font_size = round(font.pointSize()*scaling)
    # noinspection PyTypeChecker
    font.setPointSize(font_size)

    return font


def icon_to_pixmap(icon, width, height=None):
    """Converts a given icon to a pixmap. Automatically adjusts to high-DPI scaling.

    :param icon: Icon to convert.
    :param int width: Target point height.
    :param int height: Target point height.
    :return: ``QPixmap`` instance.
    """
    if not height:
        height = width

    is_hidpi = QtCore.QCoreApplication.testAttribute(Qt.AA_UseHighDpiPixmaps)
    pr = QWindow().devicePixelRatio()

    if not is_hidpi:
        width = width*pr
        height = height*pr
    px = icon.pixmap(width, height)
    if not is_hidpi:
        px.setDevicePixelRatio(pr)

    return px


class Worker(QtCore.QObject):
    """A worker object. To be used in QThreads."""

    sig_done = QtCore.pyqtSignal(object)

    def __init__(self, target=None, args=None, kwargs=None):
        QtCore.QObject.__init__(self)
        self._target = target
        self._args = args or ()
        self._kwargs = kwargs or {}

    def start(self):
        res = self._target(*self._args, **self._kwargs)
        self.sig_done.emit(res)


class BackgroundTask(QtCore.QObject):
    """A utility class to manage a worker thread."""

    sig_done = QtCore.pyqtSignal(object)

    def __init__(self, parent=None, target=None, args=None, kwargs=None, autostart=True):
        QtCore.QObject.__init__(self, parent)
        self._target = target
        self._args = args or ()
        self._kwargs = kwargs or {}

        if autostart:
            self.start()

    def start(self):

        self.thread = QtCore.QThread(self)
        self.worker = Worker(target=self._target, args=self._args, kwargs=self._kwargs)
        self.worker.sig_done.connect(self.sig_done.emit)
        self.worker.sig_done.connect(self.thread.quit)
        self.worker.moveToThread(self.thread)
        self.thread.started.connect(self.worker.start)
        self.thread.start()

    def wait(self, timeout=None):
        if timeout:
            self.thread.wait(msecs=timeout)
        else:
            self.thread.wait()


class BackgroundTaskProgressDialog(QtWidgets.QDialog):
    """A progress dialog to show during long-running background tasks."""

    def __init__(self, icon, title, message="", cancel=True, parent=None, width=450, icon_size=_USER_DIALOG_ICON_SIZE):
        super(self.__class__, self).__init__(parent=parent)
        self.setModal(True)
        self.setWindowModality(Qt.WindowModal)
        self.setWindowFlags(Qt.WindowStaysOnTopHint | Qt.Sheet | Qt.WindowTitleHint | Qt.CustomizeWindowHint)
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setWindowTitle("")
        self.setFixedWidth(width)

        self.gridLayout = QtWidgets.QGridLayout()
        self.setLayout(self.gridLayout)

        self.iconLabel = QtWidgets.QLabel(self)
        self.titleLabel = QtWidgets.QLabel(self)
        self.infoLabel = QtWidgets.QLabel(self)
        self.progressBar = QtWidgets.QProgressBar()
        self.buttonBox = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Cancel)

        self.iconLabel.setMinimumSize(icon_size, icon_size)
        self.iconLabel.setMaximumSize(icon_size, icon_size)
        self.iconLabel.setAlignment(Qt.AlignTop)
        self.titleLabel.setFont(get_scaled_font(bold=True))
        self.infoLabel.setFont(get_scaled_font(scaling=0.9))
        self.infoLabel.setFixedWidth(width-150)
        self.infoLabel.setSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        self.infoLabel.setWordWrap(True)
        self.infoLabel.setOpenExternalLinks(True)

        self.iconLabel.setPixmap(icon_to_pixmap(icon, icon_size))
        self.titleLabel.setText(title)
        self.infoLabel.setText(message)

        self.buttonBox.rejected.connect(self.reject)
        self.progressBar.setMinimum(0)
        self.progressBar.setMaximum(0)

        self.gridLayout.addWidget(self.iconLabel, 0, 0, 3, 1)
        self.gridLayout.addWidget(self.titleLabel, 0, 1, 1, 1)

        if message:
            self.gridLayout.addWidget(self.infoLabel, 1, 1, 1, 1)
            self.gridLayout.addWidget(self.progressBar, 2, 1, 1, 1)
        else:
            self.gridLayout.addWidget(self.progressBar, 1, 1, 1, 1)

        if message and cancel:
            self.gridLayout.addWidget(self.buttonBox, 3, 1, -1, -1)
        elif cancel:
            self.gridLayout.addWidget(self.buttonBox, 2, 1, -1, -1)

        self.adjustSize()


class UserDialog(QtWidgets.QDialog):
    """A template user dialog. Shows a traceback if given in constructor."""

    def __init__(self, icon, title, message, details=None, parent=None, icon_size=_USER_DIALOG_ICON_SIZE):
        super(self.__class__, self).__init__(parent=parent)
        self.setModal(True)
        self.setWindowModality(Qt.WindowModal)
        self.setWindowFlags(Qt.WindowStaysOnTopHint | Qt.Sheet | Qt.WindowTitleHint |
                            Qt.CustomizeWindowHint)
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setWindowTitle("")
        width = 550 if details else 450
        self.setFixedWidth(width)

        self.gridLayout = QtWidgets.QGridLayout()
        self.setLayout(self.gridLayout)

        self.iconLabel = QtWidgets.QLabel(self)
        self.titleLabel = QtWidgets.QLabel(self)
        self.infoLabel = QtWidgets.QLabel(self)

        self.iconLabel.setMinimumSize(icon_size, icon_size)
        self.iconLabel.setMaximumSize(icon_size, icon_size)
        self.titleLabel.setFont(get_scaled_font(bold=True))
        self.infoLabel.setFont(get_scaled_font(scaling=0.9))
        self.infoLabel.setFixedWidth(width-150)
        self.infoLabel.setSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding,
                                     QtWidgets.QSizePolicy.MinimumExpanding)
        self.infoLabel.setWordWrap(True)
        self.infoLabel.setOpenExternalLinks(True)

        self.iconLabel.setPixmap(icon_to_pixmap(icon, icon_size))
        self.titleLabel.setText(title)
        self.infoLabel.setText(message)

        if details:
            self.details = QtWidgets.QTextBrowser(self)
            self.details.setText("".join(details))
            self.details.setOpenExternalLinks(True)

        self.buttonBox = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.accepted.connect(self.accept)

        self.gridLayout.addWidget(self.iconLabel, 0, 0, 2, 1)
        self.gridLayout.addWidget(self.titleLabel, 0, 1, 1, 1)
        self.gridLayout.addWidget(self.infoLabel, 1, 1, 1, 1)
        if details:
            self.gridLayout.addWidget(self.details, 2, 1, 1, 1)
        self.gridLayout.addWidget(self.buttonBox, 3, 1, -1, -1)

        self.adjustSize()

    def setAcceptButtonName(self, name):
        self.buttonBox.buttons()[0].setText(name)

    def addCancelButton(self, name="Cancel"):
        self._cancelButton = self.buttonBox.addButton(QtWidgets.QDialogButtonBox.Cancel)
        self._cancelButton.setText(name)
        self._cancelButton.clicked.connect(self.close)

    def setCancelButtonName(self, name):
        self._cancelButton.setText(name)

    def addSecondAcceptButton(self, name, icon="dialog-ok"):
        self._acceptButton2 = self.buttonBox.addButton(QtWidgets.QDialogButtonBox.Ignore)
        self._acceptButton2.setText(name)
        if isinstance(icon, QtGui.QIcon):
            self._acceptButton2.setIcon(icon)
        elif isinstance(icon, str):
            self._acceptButton2.setIcon(QtGui.QIcon.fromTheme(icon))
        self._acceptButton2.clicked.connect(lambda: self.setResult(2))
        self._acceptButton2.clicked.connect(self.close)

    def setSecondAcceptButtonName(self, name):
        self._acceptButton2.setText(name)


def get_masked_image(path, size=64, overlay_text=""):
    """
    Returns a ``QPixmap`` from an image file masked with a smooth circle.
    The returned pixmap will have a size of *size* × *size* pixels.

    :param str path: Path to image file.
    :param int size: Target size. Will be the diameter of the masked image.
    :param overlay_text: Overlay text. This will be shown in white sans-serif on top of
        the image.
    :return: `QPixmap`` instance.
    """

    with open(path, "rb") as f:
        imgdata = f.read()

    imgtype = path.split(".")[-1]

    # Load image and convert to 32-bit ARGB (adds an alpha channel):
    image = QImage.fromData(imgdata, imgtype)
    image.convertToFormat(QImage.Format_ARGB32)

    # Crop image to a square:
    imgsize = min(image.width(), image.height())
    rect = QRect(
        (image.width() - imgsize) / 2,
        (image.height() - imgsize) / 2,
        imgsize,
        imgsize,
    )
    image = image.copy(rect)

    # Create the output image with the same dimensions and an alpha channel
    # and make it completely transparent:
    out_img = QImage(imgsize, imgsize, QImage.Format_ARGB32)
    out_img.fill(Qt.transparent)

    # Create a texture brush and paint a circle with the original image onto
    # the output image:
    brush = QBrush(image)        # Create texture brush
    painter = QPainter(out_img)  # Paint the output image
    painter.setBrush(brush)      # Use the image texture brush
    painter.setPen(Qt.NoPen)     # Don't draw an outline
    painter.setRenderHint(QPainter.Antialiasing, True)  # Use AA
    painter.drawEllipse(0, 0, imgsize, imgsize)  # Actually draw the circle

    if overlay_text:
        # draw text
        font = QtGui.QFont("Arial Rounded MT Bold")
        font.setPointSize(imgsize * 0.4)
        painter.setFont(font)
        painter.setPen(Qt.white)
        painter.drawText(QRect(0, 0, imgsize, imgsize), Qt.AlignCenter, overlay_text)

    painter.end()                # We are done (segfault if you forget this)

    # Convert the image to a pixmap and rescale it.  Take pixel ratio into
    # account to get a sharp image on retina displays:
    pr = QWindow().devicePixelRatio()
    pm = QPixmap.fromImage(out_img)
    pm.setDevicePixelRatio(pr)
    size *= pr
    pm = pm.scaled(size, size, Qt.KeepAspectRatio, Qt.SmoothTransformation)

    return pm



# !/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: Sam Schott  (ss2151@cam.ac.uk)

(c) Sam Schott; This work is licensed under a Creative Commons
Attribution-NonCommercial-NoDerivs 2.0 UK: England & Wales License.

"""

from PyQt5 import QtCore, QtWidgets


from .scientific_spinbox import ScienSpinBox, ScienDSpinBox
from .list_entry_widget import FloatListWidget


class SettingsWidget(QtWidgets.QWidget):

    def __init__(self, parent=None):
        """
        A widget to group settings for a scientific instrument or measurement.

        Settings are ordered on a grid with a QLabel which describes the setting on the
        left and a widget to modify the setting on the right. Currently supported fields
        are given below.
        """
        QtWidgets.QWidget.__init__(self, parent=parent)

        self.gridLayout = QtWidgets.QGridLayout(self)
        self.gridLayout.setVerticalSpacing(5)
        self.gridLayout.setColumnStretch(0, 1)  # first column has relative size 1
        self.gridLayout.setColumnStretch(1, 2)  # second column has relative size 2

    def addDoubleField(self, name, value, unit=None, limits=None):
        """
        Adds a setting to modify a float number.

        :param str name: Setting title. Will be displayed as label next to the spin box.
        :param float value: Initial value.
        :param str unit: Unit.
        :param limits: Tuple or list with maximum and minimum value.
        :return: Instance of :class:`scientific_spinbox.ScienDSpinBox`.
        """

        label = QtWidgets.QLabel(self)
        label.setText(name)

        spinbox = ScienDSpinBox(self)
        spinbox.setMinimumWidth(90)
        spinbox.setMaximumWidth(90)
        spinbox.setValue(value)
        if unit:
            spinbox.setSuffix(unit)
        if limits:
            spinbox.setRange(*limits)

        n_rows = self.gridLayout.rowCount()
        self.gridLayout.addWidget(label, n_rows, 0, 1, 1, alignment=QtCore.Qt.AlignRight)
        self.gridLayout.addWidget(spinbox, n_rows, 1, 1, 1, alignment=QtCore.Qt.AlignLeft)

        return spinbox

    def addIntField(self, name, value, unit=None, limits=None):
        """
        Adds a setting to modify an integer number.

        :param str name: Setting title. Will be displayed as label next to the spin box.
        :param int value: Initial value.
        :param str unit: Unit.
        :param limits: Tuple or list with maximum and minimum value.
        :return: Instance of :class:`scientific_spinbox.ScienSpinBox`.
        """
        label = QtWidgets.QLabel(self)
        label.setText(name)

        spinbox = ScienSpinBox(self)
        spinbox.setMinimumWidth(90)
        spinbox.setMaximumWidth(90)
        spinbox.setValue(value)
        if unit:
            spinbox.setSuffix(unit)
        if limits:
            spinbox.setRange(*limits)

        n_rows = self.gridLayout.rowCount()
        self.gridLayout.addWidget(label, n_rows, 0, 1, 1, alignment=QtCore.Qt.AlignRight)
        self.gridLayout.addWidget(spinbox, n_rows, 1, 1, 1, alignment=QtCore.Qt.AlignLeft)

        return spinbox

    def addSelectionField(self, name, choices, index=0):
        """
        Adds a setting to select a value from a combobox. Use this if you want to
        change the available choices programmatically at a later point.

        :param str name: Setting title. Will be displayed as label next to the combobox.
        :param list choices: List of strings that indicate choices.
        :param int index: Index of default choice.
        :return: Instance of :class:`PyQt5.QWidgets.QComboBox`.
        """
        label = QtWidgets.QLabel(self)
        label.setText(name)

        combobox = QtWidgets.QComboBox(self)
        combobox.setMinimumWidth(150)
        combobox.setMaximumWidth(150)
        combobox.addItems(choices)
        combobox.setCurrentIndex(index)

        n_rows = self.gridLayout.rowCount()
        self.gridLayout.addWidget(label, n_rows, 0, 1, 1, alignment=QtCore.Qt.AlignRight)
        self.gridLayout.addWidget(combobox, n_rows, 1, 1, 1, alignment=QtCore.Qt.AlignLeft)

        return combobox

    def addSelectionBoxes(self, name, choices, index=0):
        """
        Adds a setting to select values from mutually exclusive radio buttons. Use this if
        the set of choices will not change in the future. Use multiple checkboxes through
        :func:`addCheckBox` instead if you want to allow multiple choices to be selected.

        :param str name: Setting title. Will be displayed as label next to the combobox.
        :param list choices: List of strings that indicate choices.
        :param int index: Index of default choice.
        :return: Instance of :class:`PyQt5.QWidgets.QButtonGroup`.
        """
        label = QtWidgets.QLabel(self)
        label.setAlignment(QtCore.Qt.AlignTop)
        label.setText(name)

        button_group = QtWidgets.QButtonGroup(self)
        button_group.setExclusive(True)

        box = QtWidgets.QVBoxLayout()

        for choice, i in zip(choices, range(len(choices))):
            rb = QtWidgets.QRadioButton(choice, parent=self)
            rb.setChecked(index == i)
            button_group.addButton(rb)
            box.addWidget(rb)

        n_rows = self.gridLayout.rowCount()
        self.gridLayout.addWidget(label, n_rows, 0, 1, 1, alignment=QtCore.Qt.AlignRight)
        self.gridLayout.addLayout(box, n_rows, 1, 1, 1, alignment=QtCore.Qt.AlignLeft)

        return button_group

    def addListField(self, name, value_list):
        """
        Adds a setting to enter a list of values.

        :param name: Setting title. Will be displayed as label next to the entry widget.
        :param list value_list: Initial list of values.
        :return: Instance of :class:`list_entry_widget.FloatListWidget`.
        """

        label = QtWidgets.QLabel(self)
        label.setText(name)

        list_field = FloatListWidget(self)
        list_field.setMinimumWidth(150)
        list_field.setMaximumWidth(150)
        list_field.setValue(value_list)

        n_rows = self.gridLayout.rowCount()
        self.gridLayout.addWidget(label, n_rows, 0, 1, 1, alignment=QtCore.Qt.AlignRight)
        self.gridLayout.addWidget(list_field, n_rows, 1, 1, 1, alignment=QtCore.Qt.AlignLeft)

        return list_field

    def addCheckBox(self, name, checked=True):
        """
        Adds a check box setting.

        :param name: Setting title. Will be displayed to the right of the checkbox.
        :param bool checked: Initial checked state.
        :return: Instance of :class:`PyQt5.QWidgets.QCheckBox`.
        """

        checkbox = QtWidgets.QCheckBox(name, parent=self)
        checkbox.setChecked(checked)

        n_rows = self.gridLayout.rowCount()
        self.gridLayout.addWidget(checkbox, n_rows, 1, 1, 1, alignment=QtCore.Qt.AlignLeft)

        return checkbox

    def addSeparator(self, width=350):
        """
        Adds a horizontal line to separate sections.

        :param int width: Width in pixels.
        :return: Instance of :class:`PyQt5.QtWidgets.QFrame`.
        """

        h_line = QtWidgets.QFrame(self)
        h_line.setFrameShape(QtWidgets.QFrame.HLine)
        h_line.setFixedWidth(width)
        h_line.setStyleSheet("color: rgb(205, 203, 205)")

        n_rows = self.gridLayout.rowCount()
        self.gridLayout.addWidget(h_line, n_rows, 0, 1, -1, alignment=QtCore.Qt.AlignHCenter)

        return h_line



from setuptools import setup, find_packages

setup(
    name="mercurygui",
    version="3.0.0",
    description="A GUI to control the Oxford instruments Mercury iTC",
    author="Sam Schott",
    author_email="ss2151@cam.ac.uk",
    url="https://github.com/oe-fet/mercurygui.git",
    license="MIT",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    package_data={
        "mercurygui": ["*.ui", "*/*.ui"],
    },
    entry_points={
        "console_scripts": ["mercurygui=mercurygui.main:run"],
        "gui_scripts": ["mercurygui=mercurygui.main:run"],
    },
    install_requires=[
        "pyvisa",
        "mercuryitc>=0.3.0",
        "numpy",
        "pyqtgraph>=0.11.0",
        "PyQt5",
        "repr",
        "setuptools",
    ],
    zip_safe=False,
    keywords="mercurygui",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)
